@@ -25,7 +25,7 @@ typedef std::set<RCP<const Set>, RCPBasicKeyLess> set_set;
 class Set : public Basic
 {
 public:
-    virtual vec_basic get_args() const = 0;
+    vec_basic get_args() const override = 0;
     virtual RCP<const Set> set_intersection(const RCP<const Set> &o) const = 0;
     virtual RCP<const Set> set_union(const RCP<const Set> &o) const = 0;
     virtual RCP<const Set> set_complement(const RCP<const Set> &o) const = 0;
@@ -59,10 +59,10 @@ public:
         operator=(EmptySet const &)
         = delete;
     const static RCP<const EmptySet> &getInstance();
-    virtual hash_t __hash__() const;
-    virtual bool __eq__(const Basic &o) const;
-    virtual int compare(const Basic &o) const;
-    virtual vec_basic get_args() const
+    hash_t __hash__() const override;
+    bool __eq__(const Basic &o) const override;
+    int compare(const Basic &o) const override;
+    vec_basic get_args() const override
     {
         return {};
     }
@@ -70,10 +70,10 @@ public:
     template <typename T_, typename... Args>
     friend inline RCP<T_> make_rcp(Args &&...args);
 
-    virtual RCP<const Set> set_intersection(const RCP<const Set> &o) const;
-    virtual RCP<const Set> set_union(const RCP<const Set> &o) const;
-    virtual RCP<const Set> set_complement(const RCP<const Set> &o) const;
-    virtual RCP<const Boolean> contains(const RCP<const Basic> &a) const
+    RCP<const Set> set_intersection(const RCP<const Set> &o) const override;
+    RCP<const Set> set_union(const RCP<const Set> &o) const override;
+    RCP<const Set> set_complement(const RCP<const Set> &o) const override;
+    RCP<const Boolean> contains(const RCP<const Basic> &a) const override
     {
         return boolean(false);
     };
@@ -92,10 +92,10 @@ public:
     // UniversalSet(UniversalSet const&) = delete;
     void operator=(UniversalSet const &) = delete;
     const static RCP<const UniversalSet> &getInstance();
-    virtual hash_t __hash__() const;
-    virtual bool __eq__(const Basic &o) const;
-    virtual int compare(const Basic &o) const;
-    virtual vec_basic get_args() const
+    hash_t __hash__() const override;
+    bool __eq__(const Basic &o) const override;
+    int compare(const Basic &o) const override;
+    vec_basic get_args() const override
     {
         return {};
     }
@@ -103,10 +103,10 @@ public:
     template <typename T_, typename... Args>
     friend inline RCP<T_> make_rcp(Args &&...args);
 
-    virtual RCP<const Set> set_intersection(const RCP<const Set> &o) const;
-    virtual RCP<const Set> set_union(const RCP<const Set> &o) const;
-    virtual RCP<const Set> set_complement(const RCP<const Set> &o) const;
-    virtual RCP<const Boolean> contains(const RCP<const Basic> &a) const
+    RCP<const Set> set_intersection(const RCP<const Set> &o) const override;
+    RCP<const Set> set_union(const RCP<const Set> &o) const override;
+    RCP<const Set> set_complement(const RCP<const Set> &o) const override;
+    RCP<const Boolean> contains(const RCP<const Basic> &a) const override
     {
         return boolean(true);
     };
@@ -119,10 +119,10 @@ private:
 
 public:
     IMPLEMENT_TYPEID(SYMENGINE_FINITESET)
-    virtual hash_t __hash__() const;
-    virtual bool __eq__(const Basic &o) const;
-    virtual int compare(const Basic &o) const;
-    virtual vec_basic get_args() const
+    hash_t __hash__() const override;
+    bool __eq__(const Basic &o) const override;
+    int compare(const Basic &o) const override;
+    vec_basic get_args() const override
     {
         return vec_basic(container_.begin(), container_.end());
     }
@@ -130,10 +130,10 @@ public:
     FiniteSet(const set_basic &container);
     static bool is_canonical(const set_basic &container);
 
-    virtual RCP<const Set> set_union(const RCP<const Set> &o) const;
-    virtual RCP<const Set> set_intersection(const RCP<const Set> &o) const;
-    virtual RCP<const Set> set_complement(const RCP<const Set> &o) const;
-    virtual RCP<const Boolean> contains(const RCP<const Basic> &a) const;
+    RCP<const Set> set_union(const RCP<const Set> &o) const override;
+    RCP<const Set> set_intersection(const RCP<const Set> &o) const override;
+    RCP<const Set> set_complement(const RCP<const Set> &o) const override;
+    RCP<const Boolean> contains(const RCP<const Basic> &a) const override;
     RCP<const Set> create(const set_basic &container) const;
 
     inline const set_basic &get_container() const
@@ -151,9 +151,9 @@ private:
 
 public:
     IMPLEMENT_TYPEID(SYMENGINE_INTERVAL)
-    virtual hash_t __hash__() const;
-    virtual bool __eq__(const Basic &o) const;
-    virtual int compare(const Basic &o) const;
+    hash_t __hash__() const override;
+    bool __eq__(const Basic &o) const override;
+    int compare(const Basic &o) const override;
 
     Interval(const RCP<const Number> &start, const RCP<const Number> &end,
              const bool left_open = false, const bool right_open = false);
@@ -167,11 +167,11 @@ public:
                              const RCP<const Number> &end, bool left_open,
                              bool right_open);
 
-    virtual RCP<const Set> set_union(const RCP<const Set> &o) const;
-    virtual RCP<const Set> set_intersection(const RCP<const Set> &o) const;
-    virtual RCP<const Set> set_complement(const RCP<const Set> &o) const;
-    virtual RCP<const Boolean> contains(const RCP<const Basic> &a) const;
-    virtual vec_basic get_args() const;
+    RCP<const Set> set_union(const RCP<const Set> &o) const override;
+    RCP<const Set> set_intersection(const RCP<const Set> &o) const override;
+    RCP<const Set> set_complement(const RCP<const Set> &o) const override;
+    RCP<const Boolean> contains(const RCP<const Basic> &a) const override;
+    vec_basic get_args() const override;
 
     inline const RCP<const Number> &get_start() const
     {
@@ -203,10 +203,10 @@ public:
     IMPLEMENT_TYPEID(SYMENGINE_COMPLEXES)
     void operator=(Complexes const &) = delete;
     const static RCP<const Complexes> &getInstance();
-    virtual hash_t __hash__() const;
-    virtual bool __eq__(const Basic &o) const;
-    virtual int compare(const Basic &o) const;
-    virtual vec_basic get_args() const
+    hash_t __hash__() const override;
+    bool __eq__(const Basic &o) const override;
+    int compare(const Basic &o) const override;
+    vec_basic get_args() const override
     {
         return {};
     }
@@ -214,10 +214,10 @@ public:
     template <typename T_, typename... Args>
     friend inline RCP<T_> make_rcp(Args &&...args);
 
-    virtual RCP<const Set> set_intersection(const RCP<const Set> &o) const;
-    virtual RCP<const Set> set_union(const RCP<const Set> &o) const;
-    virtual RCP<const Set> set_complement(const RCP<const Set> &o) const;
-    virtual RCP<const Boolean> contains(const RCP<const Basic> &a) const;
+    RCP<const Set> set_intersection(const RCP<const Set> &o) const override;
+    RCP<const Set> set_union(const RCP<const Set> &o) const override;
+    RCP<const Set> set_complement(const RCP<const Set> &o) const override;
+    RCP<const Boolean> contains(const RCP<const Basic> &a) const override;
 };
 
 class Reals : public Set
@@ -232,10 +232,10 @@ public:
     IMPLEMENT_TYPEID(SYMENGINE_REALS)
     void operator=(Reals const &) = delete;
     const static RCP<const Reals> &getInstance();
-    virtual hash_t __hash__() const;
-    virtual bool __eq__(const Basic &o) const;
-    virtual int compare(const Basic &o) const;
-    virtual vec_basic get_args() const
+    hash_t __hash__() const override;
+    bool __eq__(const Basic &o) const override;
+    int compare(const Basic &o) const override;
+    vec_basic get_args() const override
     {
         return {};
     }
@@ -243,10 +243,10 @@ public:
     template <typename T_, typename... Args>
     friend inline RCP<T_> make_rcp(Args &&...args);
 
-    virtual RCP<const Set> set_intersection(const RCP<const Set> &o) const;
-    virtual RCP<const Set> set_union(const RCP<const Set> &o) const;
-    virtual RCP<const Set> set_complement(const RCP<const Set> &o) const;
-    virtual RCP<const Boolean> contains(const RCP<const Basic> &a) const;
+    RCP<const Set> set_intersection(const RCP<const Set> &o) const override;
+    RCP<const Set> set_union(const RCP<const Set> &o) const override;
+    RCP<const Set> set_complement(const RCP<const Set> &o) const override;
+    RCP<const Boolean> contains(const RCP<const Basic> &a) const override;
 };
 
 class Rationals : public Set
@@ -261,10 +261,10 @@ public:
     IMPLEMENT_TYPEID(SYMENGINE_RATIONALS)
     void operator=(Rationals const &) = delete;
     const static RCP<const Rationals> &getInstance();
-    virtual hash_t __hash__() const;
-    virtual bool __eq__(const Basic &o) const;
-    virtual int compare(const Basic &o) const;
-    virtual vec_basic get_args() const
+    hash_t __hash__() const override;
+    bool __eq__(const Basic &o) const override;
+    int compare(const Basic &o) const override;
+    vec_basic get_args() const override
     {
         return {};
     }
@@ -272,10 +272,10 @@ public:
     template <typename T_, typename... Args>
     friend inline RCP<T_> make_rcp(Args &&...args);
 
-    virtual RCP<const Set> set_intersection(const RCP<const Set> &o) const;
-    virtual RCP<const Set> set_union(const RCP<const Set> &o) const;
-    virtual RCP<const Set> set_complement(const RCP<const Set> &o) const;
-    virtual RCP<const Boolean> contains(const RCP<const Basic> &a) const;
+    RCP<const Set> set_intersection(const RCP<const Set> &o) const override;
+    RCP<const Set> set_union(const RCP<const Set> &o) const override;
+    RCP<const Set> set_complement(const RCP<const Set> &o) const override;
+    RCP<const Boolean> contains(const RCP<const Basic> &a) const override;
 };
 
 class Integers : public Set
@@ -290,10 +290,10 @@ public:
     IMPLEMENT_TYPEID(SYMENGINE_INTEGERS)
     void operator=(Integers const &) = delete;
     const static RCP<const Integers> &getInstance();
-    virtual hash_t __hash__() const;
-    virtual bool __eq__(const Basic &o) const;
-    virtual int compare(const Basic &o) const;
-    virtual vec_basic get_args() const
+    hash_t __hash__() const override;
+    bool __eq__(const Basic &o) const override;
+    int compare(const Basic &o) const override;
+    vec_basic get_args() const override
     {
         return {};
     }
@@ -301,10 +301,10 @@ public:
     template <typename T_, typename... Args>
     friend inline RCP<T_> make_rcp(Args &&...args);
 
-    virtual RCP<const Set> set_intersection(const RCP<const Set> &o) const;
-    virtual RCP<const Set> set_union(const RCP<const Set> &o) const;
-    virtual RCP<const Set> set_complement(const RCP<const Set> &o) const;
-    virtual RCP<const Boolean> contains(const RCP<const Basic> &a) const;
+    RCP<const Set> set_intersection(const RCP<const Set> &o) const override;
+    RCP<const Set> set_union(const RCP<const Set> &o) const override;
+    RCP<const Set> set_complement(const RCP<const Set> &o) const override;
+    RCP<const Boolean> contains(const RCP<const Basic> &a) const override;
 };
 
 class Naturals : public Set
@@ -319,10 +319,10 @@ public:
     IMPLEMENT_TYPEID(SYMENGINE_NATURALS)
     void operator=(Naturals const &) = delete;
     const static RCP<const Naturals> &getInstance();
-    virtual hash_t __hash__() const;
-    virtual bool __eq__(const Basic &o) const;
-    virtual int compare(const Basic &o) const;
-    virtual vec_basic get_args() const
+    hash_t __hash__() const override;
+    bool __eq__(const Basic &o) const override;
+    int compare(const Basic &o) const override;
+    vec_basic get_args() const override
     {
         return {};
     }
@@ -330,10 +330,10 @@ public:
     template <typename T_, typename... Args>
     friend inline RCP<T_> make_rcp(Args &&...args);
 
-    virtual RCP<const Set> set_intersection(const RCP<const Set> &o) const;
-    virtual RCP<const Set> set_union(const RCP<const Set> &o) const;
-    virtual RCP<const Set> set_complement(const RCP<const Set> &o) const;
-    virtual RCP<const Boolean> contains(const RCP<const Basic> &a) const;
+    RCP<const Set> set_intersection(const RCP<const Set> &o) const override;
+    RCP<const Set> set_union(const RCP<const Set> &o) const override;
+    RCP<const Set> set_complement(const RCP<const Set> &o) const override;
+    RCP<const Boolean> contains(const RCP<const Basic> &a) const override;
 };
 
 class Naturals0 : public Set
@@ -348,10 +348,10 @@ public:
     IMPLEMENT_TYPEID(SYMENGINE_NATURALS0)
     void operator=(Naturals0 const &) = delete;
     const static RCP<const Naturals0> &getInstance();
-    virtual hash_t __hash__() const;
-    virtual bool __eq__(const Basic &o) const;
-    virtual int compare(const Basic &o) const;
-    virtual vec_basic get_args() const
+    hash_t __hash__() const override;
+    bool __eq__(const Basic &o) const override;
+    int compare(const Basic &o) const override;
+    vec_basic get_args() const override
     {
         return {};
     }
@@ -359,10 +359,10 @@ public:
     template <typename T_, typename... Args>
     friend inline RCP<T_> make_rcp(Args &&...args);
 
-    virtual RCP<const Set> set_intersection(const RCP<const Set> &o) const;
-    virtual RCP<const Set> set_union(const RCP<const Set> &o) const;
-    virtual RCP<const Set> set_complement(const RCP<const Set> &o) const;
-    virtual RCP<const Boolean> contains(const RCP<const Basic> &a) const;
+    RCP<const Set> set_intersection(const RCP<const Set> &o) const override;
+    RCP<const Set> set_union(const RCP<const Set> &o) const override;
+    RCP<const Set> set_complement(const RCP<const Set> &o) const override;
+    RCP<const Boolean> contains(const RCP<const Basic> &a) const override;
 };
 
 class Union : public Set
@@ -372,17 +372,44 @@ private:
 
 public:
     IMPLEMENT_TYPEID(SYMENGINE_UNION)
-    virtual hash_t __hash__() const;
-    virtual bool __eq__(const Basic &o) const;
-    virtual int compare(const Basic &o) const;
-    virtual vec_basic get_args() const;
+    hash_t __hash__() const override;
+    bool __eq__(const Basic &o) const override;
+    int compare(const Basic &o) const override;
+    vec_basic get_args() const override;
     Union(const set_set &in);
     static bool is_canonical(const set_set &in);
 
-    virtual RCP<const Set> set_intersection(const RCP<const Set> &o) const;
-    virtual RCP<const Set> set_union(const RCP<const Set> &o) const;
-    virtual RCP<const Set> set_complement(const RCP<const Set> &o) const;
-    virtual RCP<const Boolean> contains(const RCP<const Basic> &a) const;
+    RCP<const Set> set_intersection(const RCP<const Set> &o) const override;
+    RCP<const Set> set_union(const RCP<const Set> &o) const override;
+    RCP<const Set> set_complement(const RCP<const Set> &o) const override;
+    RCP<const Boolean> contains(const RCP<const Basic> &a) const override;
+
+    inline const set_set &get_container() const
+    {
+        return this->container_;
+    }
+
+    RCP<const Set> create(const set_set &in) const;
+};
+
+class Intersection : public Set
+{
+private:
+    set_set container_;
+
+public:
+    IMPLEMENT_TYPEID(SYMENGINE_INTERSECTION)
+    hash_t __hash__() const override;
+    bool __eq__(const Basic &o) const override;
+    int compare(const Basic &o) const override;
+    vec_basic get_args() const override;
+    Intersection(const set_set &in);
+    static bool is_canonical(const set_set &in);
+
+    RCP<const Set> set_intersection(const RCP<const Set> &o) const override;
+    RCP<const Set> set_union(const RCP<const Set> &o) const override;
+    RCP<const Set> set_complement(const RCP<const Set> &o) const override;
+    RCP<const Boolean> contains(const RCP<const Basic> &a) const override;
 
     inline const set_set &get_container() const
     {
@@ -401,19 +428,19 @@ private:
 
 public:
     IMPLEMENT_TYPEID(SYMENGINE_COMPLEMENT)
-    virtual hash_t __hash__() const;
-    virtual bool __eq__(const Basic &o) const;
-    virtual int compare(const Basic &o) const;
-    virtual vec_basic get_args() const
+    hash_t __hash__() const override;
+    bool __eq__(const Basic &o) const override;
+    int compare(const Basic &o) const override;
+    vec_basic get_args() const override
     {
         return {universe_, container_};
     }
     Complement(const RCP<const Set> &universe, const RCP<const Set> &container);
 
-    virtual RCP<const Set> set_intersection(const RCP<const Set> &o) const;
-    virtual RCP<const Set> set_union(const RCP<const Set> &o) const;
-    virtual RCP<const Set> set_complement(const RCP<const Set> &o) const;
-    virtual RCP<const Boolean> contains(const RCP<const Basic> &a) const;
+    RCP<const Set> set_intersection(const RCP<const Set> &o) const override;
+    RCP<const Set> set_union(const RCP<const Set> &o) const override;
+    RCP<const Set> set_complement(const RCP<const Set> &o) const override;
+    RCP<const Boolean> contains(const RCP<const Basic> &a) const override;
 
     inline const RCP<const Set> &get_universe() const
     {
@@ -433,10 +460,10 @@ private:
 
 public:
     IMPLEMENT_TYPEID(SYMENGINE_CONDITIONSET)
-    virtual hash_t __hash__() const;
-    virtual bool __eq__(const Basic &o) const;
-    virtual int compare(const Basic &o) const;
-    virtual vec_basic get_args() const
+    hash_t __hash__() const override;
+    bool __eq__(const Basic &o) const override;
+    int compare(const Basic &o) const override;
+    vec_basic get_args() const override
     {
         return {sym, condition_};
     }
@@ -444,10 +471,10 @@ public:
                  const RCP<const Boolean> &condition);
     static bool is_canonical(const RCP<const Basic> &sym,
                              const RCP<const Boolean> &condition);
-    virtual RCP<const Set> set_intersection(const RCP<const Set> &o) const;
-    virtual RCP<const Set> set_union(const RCP<const Set> &o) const;
-    virtual RCP<const Set> set_complement(const RCP<const Set> &o) const;
-    virtual RCP<const Boolean> contains(const RCP<const Basic> &a) const;
+    RCP<const Set> set_intersection(const RCP<const Set> &o) const override;
+    RCP<const Set> set_union(const RCP<const Set> &o) const override;
+    RCP<const Set> set_complement(const RCP<const Set> &o) const override;
+    RCP<const Boolean> contains(const RCP<const Basic> &a) const override;
     inline const RCP<const Basic> &get_symbol() const
     {
         return this->sym;
@@ -468,10 +495,10 @@ private:
 
 public:
     IMPLEMENT_TYPEID(SYMENGINE_IMAGESET)
-    virtual hash_t __hash__() const;
-    virtual bool __eq__(const Basic &o) const;
-    virtual int compare(const Basic &o) const;
-    virtual vec_basic get_args() const
+    hash_t __hash__() const override;
+    bool __eq__(const Basic &o) const override;
+    int compare(const Basic &o) const override;
+    vec_basic get_args() const override
     {
         return {sym_, expr_, base_};
     }
@@ -481,10 +508,10 @@ public:
     static bool is_canonical(const RCP<const Basic> &sym,
                              const RCP<const Basic> &expr,
                              const RCP<const Set> &base);
-    virtual RCP<const Set> set_intersection(const RCP<const Set> &o) const;
-    virtual RCP<const Set> set_union(const RCP<const Set> &o) const;
-    virtual RCP<const Set> set_complement(const RCP<const Set> &o) const;
-    virtual RCP<const Boolean> contains(const RCP<const Basic> &a) const;
+    RCP<const Set> set_intersection(const RCP<const Set> &o) const override;
+    RCP<const Set> set_union(const RCP<const Set> &o) const override;
+    RCP<const Set> set_complement(const RCP<const Set> &o) const override;
+    RCP<const Boolean> contains(const RCP<const Basic> &a) const override;
 
     inline const RCP<const Basic> &get_symbol() const
     {
@@ -519,6 +546,7 @@ inline bool is_a_Set(const Basic &b)
             || b.get_type_code() == SYMENGINE_NATURALS
             || b.get_type_code() == SYMENGINE_NATURALS0
             || b.get_type_code() == SYMENGINE_UNION
+            || b.get_type_code() == SYMENGINE_INTERSECTION
             || b.get_type_code() == SYMENGINE_IMAGESET);
 }
 
